#!/usr/bin/env python
# -*- coding: utf-8 -*-

from google_images_download import google_images_download  
import sys, io
from urllib.request import urlopen
import seaborn as sns
import matplotlib.pyplot as plt
from datetime import datetime
from requests.exceptions import HTTPError
from urllib.error import HTTPError

"""Helper functions for the emotapal package"""

def parse_color(clr):
	"""Returns an RGB list if color is RGB, and converts a hex color to RGB otherwise."""
	if type(clr) == str:
		return hex2rgb(clr)
	elif type(clr) == list:
		return clr
	elif type(clr) == tuple:
		return list(clr)

def hex2rgb(color):
	"""Converts a hex str to RGB list."""
	rgb_clr = list(tuple(bytes.fromhex("{}".format(color.strip("#")))))
	return rgb_clr

def rgb2hex(color):
	"""Converts an RGB tupple or list to a hex string."""
	hex_clr = "#" + bytes(color).hex()
	return hex_clr

def read_web_image(url):
	"""Tries to read in an image via url."""
	result = "Failed"
	try:
		result = io.BytesIO(urlopen(url).read())
	except HTTPError as e:
		print(e)
	except ValueError as e:
		print("Bad url given")
	except Exception as e:
		print("Could not read image")
	return result 

def get_gimg_urls(search_term, n):
	"""Returns n Google Image urls for a given search term."""
	f = write_gimg_urls(search_term, n)
	urls = parse_gimg_urls(f)
	return urls

def write_gimg_urls(search_term, n):
	"""Write N Google Image urls for a search term S to a text file."""
	f = open('URLS.txt', 'w') # Open text file to write print output to
	orig_stdout, sys.stdout = sys.stdout, f # Keep location of original sys.stdout  
	response = google_images_download.googleimagesdownload() # Get images
	arguments = {"keywords":search_term + " –no-download" , "limit":n, "print_urls":True} #Don't download, but print urls
	paths = response.download(arguments) 
	sys.stdout = orig_stdout # Switch to normal print output, close file
	f.close()
	return f

def parse_gimg_urls(urls):
	"""Parses output of Gogle Image results to return urls."""
	with open('URLS.txt') as f: content = f.readlines()
	urls = [content[j-1][11:-1] for j in range(len(content)) if content[j][:9] == 'Completed']
	return urls    


def label_palette(clrs, ax, save_img):
	"""Labels a custom color palette"""
	for i, name in enumerate(clrs):
		ax.text(i, 0.5, name, 
			fontsize= 8 * int(3.0/len(clrs)+1), 
			bbox= dict(facecolor='white', alpha=0.75), 
			ha="center", va="center"
		) 
	if save_img: 
		dt = datetime.now().strftime("%m.%d.%Y.%H:%M:%S")
		plt.tight_layout()
		plt.savefig("{}.png".format(dt), dpi=300, bbox_inches="tight")
	return ax



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from emotapal import EmotaPal 
from emotapal.helpers import hex2rgb, rgb2hex, parse_color 
import unittest

class TestType(unittest.TestCase):
	"""Test that each constructor method really returns an EmotaPal instance."""


	gimg = EmotaPal().from_gimg("happy", 5)

	col = EmotaPal().from_colors([[100, 200, 100], [100, 100, 100]])

	url = EmotaPal().from_url("https://slack-files2.s3-us-west-2.amazonaws.com/avatars/2017-12-19/288981919427_f45f04edd92902a96859_512.png", 4)

	# Test the `from_gimg` constructor 
	def test_from_gimg(self):
		self.assertIsInstance(self.gimg, EmotaPal)

	# Test the `from_colors` constructor 
	def test_from_col(self):
		self.assertIsInstance(self.col, EmotaPal)
	
	# The `from_url` constructor calls the `from_img` constructor after reading url,
	# so if `from_url` returns an EmotaPal then `from_img` returns and EmotaPal. 
	def test_from_url(self):
		self.assertIsInstance(self.col, EmotaPal)

class TestConstructErrors(unittest.TestCase):
	"""Test that errors are thrown when a user incorrectly uses an EmotaPal constructor."""

	# Throw an error when user enters a bad (non-HEX) string.
	def test_con1(self):
		with self.assertRaises(Exception): a = EmotaPal().from_colors(["a string"])
	
	# Throw an error when user enters a single color, not a list.
	def test_con2(self):
		with self.assertRaises(Exception): a = EmotaPal().from_colors((100, 200, 100))

	# Throw an error when user enters a bad image file. 
	def test_con3(self):
		with self.assertRaises(Exception): a = EmotaPal().from_image("joshiscool")

class TestHelperFuncs(unittest.TestCase):
	"""Test that errors are thrown when a user incorrectly uses an EmotaPal constructor."""

	hex_col = "#ffffff"
	rgb_col = [255, 255, 255]

	# Verify hex2rgb returns a list. 
	def test_hex2rgb(self):
		self.assertIsInstance(hex2rgb(self.hex_col), list)
	
	# Verify rgb2hex returns a string. 
	def test_rgb2hex(self):
		self.assertIsInstance(rgb2hex(self.rgb_col), str)

	# Verify parse_color correctly parses hex colors. 
	def test_parse_color1(self):
		self.assertListEqual(hex2rgb(self.hex_col), self.rgb_col)

	# Verify parse_color correctly parses rgb colors. 
	def test_parse_color2(self):
		self.assertEqual(rgb2hex(self.rgb_col), self.hex_col)

if __name__ == '__main__':
    unittest.main()



from distutils.core import setup

setup(
  name = 'emotapal',
  packages = ['emotapal'],   
  version = '0.2',      
  license='gpl-3.0',        
  description = 'Match colors and words',   
  author = 'Josh Ashkinaze',                   
  author_email = 'josh.ashkinaze@gmail.com',   
  url = 'https://github.com/josh-ashkinaze/emotapal',   
  download_url = 'https://github.com/josh-ashkinaze/emotapal/archive/v_02.tar.gz',    
  keywords = ['colors', 'emotions', 'psychology', 'image processing'],   # Keywords that define your package best
  install_requires=[           
      "requests", "scikit-learn", "seaborn",
      "matplotlib", "colorthief", "google_images_download", "afinn"
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',     
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   
    'Programming Language :: Python :: 3',      #
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
  ],
)






#!/usr/bin/env python
# -*- coding: utf-8 -*-

from colorthief import ColorThief # grab color info 
from sklearn.externals import joblib # open our classifier
from collections import OrderedDict # find unique list values, maintaining order
import emotapal.helpers as helpers # helper functions 
import seaborn as sns # make color palette
import matplotlib.pyplot as plt # assist with color palette
import os, sys # point to the right places
from afinn import Afinn # compute sentiment

"""
         .        .  
  * _  __|_  _. __|_  
  |(_)_) [ )(_]_) [ )
._|

"""					

class EmotaPal():
	""" 
	An EmotaPal is a palette of words and colors. 
	Just as a 'color palette' describes the colors of some visual object, 
	an 'emotion palette' describes the emotions felt from a visual object. 
	An EmotaPal combines both pieces (visual and psychological) of information. 

	Attributes: 
		topn (int): Return only the topn best color-emotion matches 
		unique_words (bool): Return unique list of words (sometimes a word is matched by >1 colors in a palette)
		_info (list): The actual palette information (emotions, colors, and distance to emotion-color match) as a list
		of dictionaries. By default, this list sorted by distance in ascending order. 
		clf (pkl): A KNN classifier used to predict the emotion of a color 


	Upon instantiation, each EmotaPal stores a WordPal and ColorPal object as properties. 
	"""
	
	def __init__(self, topn=100, unique_words = False, info=None ):
		self.topn = topn  
		self.unique_words  = unique_words
		self._info = info 
		self.clf =  joblib.load(os.path.join(os.path.dirname(os.path.abspath(__file__)), "clf.pkl")) 
	
	@property
	def words(self):
		"""Instantiate a WordPal object based on EmotaPal's words"""
		return WordPal([x['emotion'] for x in self._info], self.unique_words)
	
	@property
	def colors(self):
		"""Instantiate a ColorPal object based on EmotaPal's colors"""
		return  ColorPal([x['color'] for x in self._info])

	@property 
	def info(self):
		"""Returns the information of the palette"""
		return self._info 

	
	def from_gimg(self, query, nimages):
		"""
		Returns an EmotaPal from a gimg search. 

		To return an EmotaPal, this function first finds the url
		top nimages for a query. This method then returns
		the dominant color of each image. The resulting color set 
		is treated as a color palette, and is fed into the from_colors
		constructor method. 

		Args:
			query (str): keywords to search google images for 
			nimages (int): number of images to attempt to fetch  

		Returns:
			EmotaPal object 
		"""
		urls = helpers.get_gimg_urls(query, nimages)
		imgs = [helpers.read_web_image(u) for u in urls]
		clrs = [ColorThief(i).get_color(quality=1) for i in imgs if i != "Failed"]
		assert (len(clrs) > 0), "Could not parse any urls :("
		return self.from_colors(clrs)

	def from_url(self, url, ncolors):
		"""Constructs an EmotaPal from a url."""
		img = helpers.read_web_image(url)
		if img != "Failed":
			return self.from_image(img, ncolors)
	
	def from_image(self, img, ncolors):
		"""Constructs an EmotaPal from an image"""
		colorthief = ColorThief(img)
		if ncolors == 1:
			clr =  colorthief.get_color(quality=1)
			return self.from_colors([clr])   
		elif ncolors > 1:
			clrs =  colorthief.get_palette(color_count=ncolors, quality=1)
			return self.from_colors(clrs)  
	
	def from_colors(self, clrs):
		"""Constructs an EmotaPal from a set of colors."""
		try:
			matches = [self.nearest_emotion(c) for c in clrs]
			self._info = self.parse_matches(matches)
			return self 
		except ValueError:
			raise ValueError("Could not parse colors. Please enter a list of colors.")


	def nearest_emotion(self, clr):
		"""
		Returns information on the nearest emotion to a color. 

		To find the "nearest emotion", we predict the emotion of 
		a color with a KNN classifier trained on a dataset with features 
		["Emotion", "Dominant Color"].  

		Args:
			self: self  
			color (list, tupple, or hex): an input color 

		Returns:
			dict: {
					"emotion": nearest emotion to an input color, 
					"distance": distance from input color to emotion color, 
					"color": input color
		}
		
		"""
		color = helpers.parse_color(clr)
		emotion = self.clf.predict([color])[0]
		distance = self.clf.kneighbors([color])[0][0][0]
		results =  {"emotion":emotion, "distance":distance, "color":color}
		return results

	def parse_matches(self, data):
		""" Returns the topn matches by shortest distance to input color. """
		top_matches = sorted(data, key=lambda x: x['distance'])[:self.topn]
		return top_matches

class WordPal():

	"""
	A WordPal is a list of words associated with a list of colors. 

	Attributes:
		_text (list): a list of words 

	"""

	def __init__(self, word_list=None, unique=False):
		self._text = word_list 
		self.unique =unique
	
	@property
	def sentiment(self):
		"""
		Returns the sentiment of a WordPal word list.

		Sentiment is computed by Affin, which ranks words
		on a neg/pos scale of [-5, 5]. 

		Returns:
			Sum of sentiment scores for each word in self.text 
		"""
		sentiment = Afinn().score(" ".join(self._text))
		return sentiment
	
	@property 
	def text(self):
		"""
		Returns the text of a word palette

		
		"""
		if self.unique:
			return list(OrderedDict.fromkeys(self._text))
		else:
			return self._text

class ColorPal():
	
	"""
	A ColorPal is a list of colors associated with a list of words. 

	Each ColorPal contains information about its colors, as well as a method
	for displaying itself as a color palette. 

	Attributes:
		colors (list): a list of colors represented `[R, G, B]` values
	"""
	
	def __init__(self, color_list=None):
		self._colors = color_list
	
	@property
	def as_rgb(self):
		"""Returns a list of colors, each color a lists of RGB values."""
		return self._colors

	@property 
	def as_hex(self):
		"""Returns a list of colors, each color HEX string."""
		return [helpers.rgb2hex(c) for c in self._colors]

	def display(self, save_img=False):
		"""
		Displays a color palette of ColorPal colors.  

		Note that colors are displayed in descending order of best match. 
		So the first color is the color with the shortest distance to an emotion, etc. 
		"""

		clrs = self.as_hex
		sns.set(context="poster") # Make big 
		sns.palplot(clrs) # Create color palette 
		labels = helpers.label_palette(clrs, plt.gca(), save_img) 
		plt.show()
		plt.close()


